@@ -2,6 +2,27 @@ import os
 
 from . import model
 from . import files
+from .dsloader import DatasetLoader
+from .dssaver import DatasetSaver
+from .schema import Schema, NS_MAP
+from . import utils
+
+
+def load_dataset(base_dir: str):
+    schema = Schema()
+    loader = DatasetLoader(schema)
+    ds = loader.load(base_dir)
+    return ds
+
+
+def save_dataset(ds: model.Dataset, target_dir: str):
+    saver = DatasetSaver(ds._schema)
+    return saver.save(ds, target_dir)
+
+
+def to_etree(ds: model.Dataset):
+    return ds.to_etree(nsmap_=NS_MAP)
+
 
 # start monkey-patching generated code
 from .query import XPathQuery
@@ -42,6 +63,7 @@ def _folder_get_relative_path(folder: model.Folder):
 
 setattr(model.Folder, 'get_relative_path', _folder_get_relative_path)
 
+
 def _file_get_relative_path(file: model.File):
     return _get_path(file.parent_object_, file.name, False)
 
@@ -57,13 +79,13 @@ def _get_path(folder: model.Folder, file_name, absolute=True):
     while current_folder is not None:
         if isinstance(current_folder, model.Dataset):
             if absolute:
-                segments.insert(0, current_folder.get_base())
+                segments.insert(0, current_folder.base_dir_)
             # assume we reached the highest level, maybe not good for nested datasets
             break
         else:
             segments.insert(0, current_folder.get_name())
         current_folder = current_folder.parent_object_
-    _path = os.path.join(*segments)
+    _path = os.path.join(*segments) if segments else ''
     return _path
 
 
@@ -110,6 +132,7 @@ setattr(model.Folder, 'get_folders_sorted', get_folders_sorted)
 
 # end monkey-patching
 
+
 from .pybids_compat import BIDSLayout
 
 from . import _version
